import type * as ts from 'typescript/lib/tsserverlibrary';
import type * as embedded from '@volar/language-core';
export declare function getProgram(ts: typeof import('typescript/lib/tsserverlibrary'), core: embedded.LanguageContext, ls: ts.LanguageService, sys: ts.System): ts.Program;
