"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.register = void 0;
function register(htmlLs) {
    return (pugDoc, pos, options) => {
        const htmlStart = pugDoc.map.toGeneratedPosition(pos);
        if (!htmlStart)
            return;
        const text = htmlLs.doQuoteComplete(pugDoc.htmlTextDocument, htmlStart, pugDoc.htmlDocument, options);
        return text;
    };
}
exports.register = register;
//# sourceMappingURL=quoteComplete.js.map