import * as vscode from 'vscode-languageserver';
import { CancellationTokenHost } from '../cancellationPipe';
import type { Workspaces } from '../workspaces';
import { RuntimeEnvironment, InitializationOptions } from '../../types';
import { createDocuments } from '../documents';
export declare function register(connection: vscode.Connection, projects: Workspaces, initParams: vscode.InitializeParams, initOptions: InitializationOptions, cancelHost: CancellationTokenHost, semanticTokensLegend: vscode.SemanticTokensLegend, runtime: RuntimeEnvironment, documents: ReturnType<typeof createDocuments>): void;
export declare function sleep(ms: number): Promise<unknown>;
