import { Service } from '@volar/language-service';
import * as html from 'vscode-html-languageservice';
import { TextDocument } from 'vscode-languageserver-textdocument';
import { VueCompilerOptions } from '../types';
declare const _default: <S extends Service>(options: {
    getScanner(service: ReturnType<S>, document: TextDocument): html.Scanner | undefined;
    updateCustomData(service: ReturnType<S>, extraData: html.IHTMLDataProvider[]): void;
    baseService: S;
    isSupportedDocument: (document: TextDocument) => boolean;
    vueCompilerOptions: VueCompilerOptions;
}) => Service;
export default _default;
