import type * as vscode from 'vscode-languageserver-protocol';
import { SharedContext } from '../types';
export declare function register(ctx: SharedContext): (uri: string, position: vscode.Position) => vscode.LocationLink[];
