import type * as ts from 'typescript/lib/tsserverlibrary';
export declare namespace SymbolKind {
    function fromProtocolScriptElementKind(kind: ts.ScriptElementKind): 2 | 5 | 10 | 22 | 11 | 6 | 7 | 13 | 12 | 9 | 26 | 15;
}
