import * as CompilerDom from '@vue/compiler-dom';
export * from '@vue/compiler-dom';
export declare function compile(template: string, options?: CompilerDom.CompilerOptions): CompilerDom.CodegenResult;
