import type { Service } from '@volar/language-service';
import * as css from 'vscode-css-languageservice';
import type { TextDocument } from 'vscode-languageserver-textdocument';
export interface Provide {
    'css/stylesheet': (document: TextDocument) => css.Stylesheet | undefined;
    'css/languageService': (languageId: string) => css.LanguageService | undefined;
}
declare const _default: () => Service<Provide>;
export default _default;
