import type * as Proto from '../protocol';
export declare function snippetForFunctionCall(item: {
    insertText?: string;
    label: string;
}, displayParts: ReadonlyArray<Proto.SymbolDisplayPart>): {
    snippet: string;
    parameterCount: number;
};
