export declare const variableDeclaredButNeverUsed: Set<number>;
export declare const propertyDeclaretedButNeverUsed: Set<number>;
export declare const allImportsAreUnused: Set<number>;
export declare const unreachableCode: Set<number>;
export declare const unusedLabel: Set<number>;
export declare const fallThroughCaseInSwitch: Set<number>;
export declare const notAllCodePathsReturnAValue: Set<number>;
export declare const incorrectlyImplementsInterface: Set<number>;
export declare const cannotFindName: Set<number>;
export declare const extendsShouldBeImplements: Set<number>;
export declare const asyncOnlyAllowedInAsyncFunctions: Set<number>;
