export * from '@volar/language-service';
export * from '@vue/language-core';
export * from './ideFeatures/nameCasing';
export * from './languageService';
export { TagNameCasing, AttrNameCasing } from './types';
export { Provide } from './plugins/vue';
