import type { Service } from '@volar/language-service';
import type * as html from 'vscode-html-languageservice';
import { TextDocument } from 'vscode-languageserver-textdocument';
import * as pug from './languageService';
export interface Provide {
    'pug/pugDocument': (document: TextDocument) => pug.PugDocument | undefined;
    'pug/languageService': () => pug.LanguageService;
    'pug/updateCustomData': (extraData: html.IHTMLDataProvider[]) => void;
}
declare const _default: () => Service<Provide>;
export default _default;
