"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TypeMeta = void 0;
var TypeMeta;
(function (TypeMeta) {
    TypeMeta[TypeMeta["Unknown"] = 0] = "Unknown";
    TypeMeta[TypeMeta["Class"] = 1] = "Class";
    TypeMeta[TypeMeta["Function"] = 2] = "Function";
})(TypeMeta || (exports.TypeMeta = TypeMeta = {}));
//# sourceMappingURL=types.js.map