import type * as vscode from 'vscode-languageserver-protocol';
export declare function transform(symbol: vscode.WorkspaceSymbol, getOtherLocation: (location: vscode.Location) => vscode.Location | undefined): vscode.WorkspaceSymbol | undefined;
