"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getSlotsPropertyName = void 0;
function getSlotsPropertyName(vueVersion) {
    return vueVersion < 3 ? '$scopedSlots' : '$slots';
}
exports.getSlotsPropertyName = getSlotsPropertyName;
//# sourceMappingURL=shared.js.map