import type * as ts from 'typescript/lib/tsserverlibrary';
export default function (ts: typeof import('typescript/lib/tsserverlibrary'), host: ts.LanguageServiceHost, service: ts.LanguageService): void;
