"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
console.warn('volar-service-emmet: this module is not yet supported for web.');
exports.default = () => () => ({});
//# sourceMappingURL=empty.js.map