import * as vscode from 'vscode-languageserver';
import type { Workspaces } from '../workspaces';
import { RuntimeEnvironment } from '../../types';
export declare function register(connection: vscode.Connection, workspaces: Workspaces, env: RuntimeEnvironment): void;
