import * as html from 'vscode-html-languageservice';
export declare function loadTemplateData(lang: string): html.HTMLDataV1;
export declare function loadLanguageBlocks(lang: string): html.HTMLDataV1;
export declare function loadModelModifiersData(lang: string): html.HTMLDataV1;
