import * as vscode from 'vscode-languageserver/node';
import { InitializationOptions, LanguageServerPlugin } from '../types';
import { FileSystem } from '@volar/language-service';
export * from '../index';
export declare const uriToFileName: (uri: string) => string;
export declare const fileNameToUri: (fileName: string) => string;
export declare function createFs(options: InitializationOptions): FileSystem;
export declare function createConnection(): vscode._Connection<vscode._, vscode._, vscode._, vscode._, vscode._, vscode._, vscode._, vscode._>;
export declare function startLanguageServer(connection: vscode.Connection, ...plugins: LanguageServerPlugin[]): void;
