"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const browser_1 = require("@volar/language-server/browser");
const languageServerPlugin_1 = require("./languageServerPlugin");
const connection = (0, browser_1.createConnection)();
const plugin = (0, languageServerPlugin_1.createServerPlugin)(connection);
(0, browser_1.startLanguageServer)(connection, plugin);
//# sourceMappingURL=webServer.js.map