import type { Service } from '@volar/language-service';
import { TextDocument } from 'vscode-languageserver-textdocument';
import * as vue from '@vue/language-core';
export interface Provide {
    'vue/vueFile': (document: TextDocument) => vue.VueFile | undefined;
}
declare const _default: () => Service<Provide>;
export default _default;
