export * from '@volar/language-core';
export * from './baseLanguageService';
export * from './documents';
export { mergeWorkspaceEdits } from './languageFeatures/rename';
export * from './types';
export * as transformer from './transformer';
import type * as vscode from 'vscode-languageserver-protocol';
export declare const standardSemanticTokensLegend: vscode.SemanticTokensLegend;
