"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SymbolKind = void 0;
const PConst = __importStar(require("../protocol.const"));
var SymbolKind;
(function (SymbolKind) {
    function fromProtocolScriptElementKind(kind) {
        switch (kind) {
            case PConst.Kind.module: return 2;
            case PConst.Kind.class: return 5;
            case PConst.Kind.enum: return 10;
            case PConst.Kind.enumMember: return 22;
            case PConst.Kind.interface: return 11;
            case PConst.Kind.indexSignature: return 6;
            case PConst.Kind.callSignature: return 6;
            case PConst.Kind.method: return 6;
            case PConst.Kind.memberVariable: return 7;
            case PConst.Kind.memberGetAccessor: return 7;
            case PConst.Kind.memberSetAccessor: return 7;
            case PConst.Kind.variable: return 13;
            case PConst.Kind.let: return 13;
            case PConst.Kind.const: return 13;
            case PConst.Kind.localVariable: return 13;
            case PConst.Kind.alias: return 13;
            case PConst.Kind.function: return 12;
            case PConst.Kind.localFunction: return 12;
            case PConst.Kind.constructSignature: return 9;
            case PConst.Kind.constructorImplementation: return 9;
            case PConst.Kind.typeParameter: return 26;
            case PConst.Kind.string: return 15;
            default: return 13;
        }
    }
    SymbolKind.fromProtocolScriptElementKind = fromProtocolScriptElementKind;
})(SymbolKind || (exports.SymbolKind = SymbolKind = {}));
//# sourceMappingURL=typeConverters.js.map