"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ParseSFCRequest = exports.GetConvertAttrCasingEditsRequest = exports.GetConvertTagCasingEditsRequest = exports.DetectNameCasingRequest = exports.GetComponentMeta = void 0;
const vscode = require("vscode-languageserver-protocol");
var GetComponentMeta;
(function (GetComponentMeta) {
    GetComponentMeta.type = new vscode.RequestType('vue/componentMeta');
})(GetComponentMeta || (exports.GetComponentMeta = GetComponentMeta = {}));
var DetectNameCasingRequest;
(function (DetectNameCasingRequest) {
    DetectNameCasingRequest.type = new vscode.RequestType('vue/detectTagCasing');
})(DetectNameCasingRequest || (exports.DetectNameCasingRequest = DetectNameCasingRequest = {}));
var GetConvertTagCasingEditsRequest;
(function (GetConvertTagCasingEditsRequest) {
    GetConvertTagCasingEditsRequest.type = new vscode.RequestType('vue/convertTagNameCasing');
})(GetConvertTagCasingEditsRequest || (exports.GetConvertTagCasingEditsRequest = GetConvertTagCasingEditsRequest = {}));
var GetConvertAttrCasingEditsRequest;
(function (GetConvertAttrCasingEditsRequest) {
    GetConvertAttrCasingEditsRequest.type = new vscode.RequestType('vue/convertPropNameCasing');
})(GetConvertAttrCasingEditsRequest || (exports.GetConvertAttrCasingEditsRequest = GetConvertAttrCasingEditsRequest = {}));
var ParseSFCRequest;
(function (ParseSFCRequest) {
    ParseSFCRequest.type = new vscode.RequestType('vue/parseSfc');
})(ParseSFCRequest || (exports.ParseSFCRequest = ParseSFCRequest = {}));
//# sourceMappingURL=protocol.js.map