import { Config } from '@volar/language-service';
import * as vue from '@vue/language-core';
import type * as ts from 'typescript/lib/tsserverlibrary';
import createJsonService from 'volar-service-json';
export interface Settings {
    json?: Parameters<typeof createJsonService>[0];
}
export declare function resolveConfig(config: Config, compilerOptions?: ts.CompilerOptions, vueCompilerOptions?: Partial<vue.VueCompilerOptions>, ts?: typeof import('typescript/lib/tsserverlibrary'), settings?: Settings, codegenStack?: boolean): Config;
