import * as vscode from 'vscode-languageserver';
export type CancellationTokenHost = ReturnType<typeof createCancellationTokenHost>;
export declare function createCancellationTokenHost(_cancellationPipeName: string | undefined): {
    createCancellationToken(original?: vscode.CancellationToken): vscode.CancellationToken;
};
