export declare const enum Chars {
    /** `#` character */
    Hash = 35,
    /** `$` character */
    Dollar = 36,
    /** `-` character */
    Dash = 45,
    /** `.` character */
    Dot = 46,
    /** `:` character */
    Colon = 58,
    /** `,` character */
    Comma = 44,
    /** `!` character */
    Excl = 33,
    /** `@` character */
    At = 64,
    /** `%` character */
    Percent = 37,
    /** `_` character */
    Underscore = 95,
    /** `(` character */
    RoundBracketOpen = 40,
    /** `)` character */
    RoundBracketClose = 41,
    /** `{` character */
    CurlyBracketOpen = 123,
    /** `}` character */
    CurlyBracketClose = 125,
    /** `+` character */
    Sibling = 43,
    /** `'` character */
    SingleQuote = 39,
    /** `"` character */
    DoubleQuote = 34,
    /** `t` character */
    Transparent = 116,
    /** `/` character */
    Slash = 47
}
