import type { AbbreviationNode } from '@emmetio/abbreviation';
import type { Container } from '../utils.js';
import type { Config } from '../../config.js';
export default function lorem(node: AbbreviationNode, ancestors: Container[], config: Config): void;
