import type { AbbreviationNode } from '@emmetio/abbreviation';
/**
 * XSL transformer: removes `select` attributes from certain nodes that contain
 * children
 */
export default function xsl(node: AbbreviationNode): void;
