import { AutoInsertionContext, Service } from '@volar/language-service';
import type * as ts from 'typescript/lib/tsserverlibrary';
import type { TextDocument } from 'vscode-languageserver-textdocument';
declare const _default: () => Service;
export default _default;
export declare function isCharacterTyping(document: TextDocument, options: AutoInsertionContext): boolean;
export declare function isBlacklistNode(ts: typeof import('typescript/lib/tsserverlibrary'), node: ts.Node, pos: number, allowAccessDotValue: boolean): boolean;
