import type { Service } from '@volar/language-service';
import * as json from 'vscode-json-languageservice';
import type { TextDocument } from 'vscode-languageserver-textdocument';
export interface Provide {
    'json/jsonDocument': (document: TextDocument) => json.JSONDocument | undefined;
    'json/languageService': () => json.LanguageService;
}
declare const _default: (settings?: json.LanguageSettings) => Service<Provide>;
export default _default;
