import * as vscode from 'vscode-languageserver/browser';
import { LanguageServerPlugin } from '../types';
export * from '../index';
export declare function createConnection(): vscode.Connection;
export declare function startLanguageServer(connection: vscode.Connection, ...plugins: LanguageServerPlugin[]): void;
