import type * as vscode from 'vscode-languageserver-protocol';
export declare function transform(ranges: vscode.FoldingRange[], getOtherRange: (range: vscode.Range) => vscode.Range | undefined): vscode.FoldingRange[];
