export declare enum TagNameCasing {
    Kebab = 0,
    Pascal = 1
}
export declare enum AttrNameCasing {
    Kebab = 0,
    Camel = 1
}
export * from '@volar/language-service/out/types';
export * from '@vue/language-core/out/types';
